#include <QApplication>
#include <QMainWindow>
#include <QVBoxLayout>
#include <QDebug>
#include <QTabletEvent>
#include <QMenu>
#include <QMenuBar>
#include <QAction>
#include <QLabel>

#include <algorithm>

static bool optTracking = true;
static bool optLogMouseMove = false;

static QString description()
{
    QString t = QT_VERSION_STR;
#if QT_VERSION >= 0x050000
    t += QLatin1Char(' ') + QGuiApplication::platformName();
#endif
    return t;
}

class TabletEventFilter : public QObject
{
public:
    explicit TabletEventFilter(QObject *parent) : QObject(parent) { }

    bool eventFilter(QObject *o, QEvent *event) override
    {
        switch (event->type()) {
        case QEvent::TabletEnterProximity:
        case QEvent::TabletLeaveProximity:
        case QEvent::TabletPress:
        case QEvent::TabletMove:
        case QEvent::TabletRelease:
        case QEvent::TabletTrackingChange:
        case QEvent::MouseButtonDblClick:
        case QEvent::MouseButtonPress:
        case QEvent::MouseButtonRelease:
            qDebug().noquote()<< o->objectName() << event;
            break;
        case QEvent::MouseMove:
            if (optLogMouseMove)
                qDebug().noquote()<< o->objectName() << event;
            break;
        default:
            break;
        }
        return false;
    }
};

class TestWidget : public QWidget
{
    Q_OBJECT
public:
    TestWidget(QWidget *parent = 0);
protected:
    void tabletEvent(QTabletEvent *event) override;
    void enterEvent(QEvent *event) override;
    void leaveEvent(QEvent *event) override;
};

TestWidget::TestWidget(QWidget *parent) : QWidget(parent)
{
    setObjectName("TestWidget");
    qDebug() << "setTabletTracking" << optTracking;
    if (optTracking)
        setTabletTracking(true);
}

void TestWidget::tabletEvent(QTabletEvent *event)
{
    qDebug() << "tabletEvent: " << event;
}

void TestWidget::enterEvent(QEvent *event)
{
    qDebug() << "enterEvent: " << event;
}

void TestWidget::leaveEvent(QEvent *event)
{
    qDebug() << "leaveEvent: " << event;
}

int main(int argc, char** argv)
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

#if QT_VERSION >= 0x050600
    if (args.contains("-n")) {
        qDebug() << "DisableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
    }
#endif

    QApplication app(argc, argv);
    if (args.contains("-e"))
        app.installEventFilter(new TabletEventFilter(&app));
    optTracking = !args.contains("-t");
    optLogMouseMove = args.contains("-m");

    QMainWindow mainWindow;
    QAction *qa = mainWindow.menuBar()->addMenu("File")->addAction("Quit", qApp, &QCoreApplication::quit);
    qa->setShortcut(Qt::CTRL + Qt::Key_Q);

    QString t = description();
    qDebug() << t;
    mainWindow.setWindowTitle(t);

    TestWidget *testWidget = new TestWidget;

    QLabel *label = new QLabel;
    label->setText("Move the cursor from the TestWidget below into this Label");
    label->setStyleSheet("QLabel{ border: 2px solid green; }");

    QWidget *central = new QWidget;
    QVBoxLayout *vLayout = new QVBoxLayout(central);
    vLayout->addWidget(label);
    vLayout->addWidget(testWidget);

    mainWindow.setCentralWidget(central);
    mainWindow.show();

    return app.exec();
}
#include "main.moc"
